package com.example.springboot.controller;

import com.example.springboot.entity.RestBean;
import com.example.springboot.service.AuthorizeService;
import org.hibernate.validator.constraints.Length;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpSession;
import javax.validation.constraints.Pattern;

@Validated
@RestController
@RequestMapping("/api/auth")
public class AuthorizeController {
    private final String EMAIL_REGEXP = "^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+.[a-zA-Z]{2,}$";
    private final String USERNAME_REGEXP = "^[a-zA-Z0-9\\u4E00-\\u9FA5]+$";

    @Resource
    AuthorizeService service;

    /*
     * 新用户注册发送邮箱验证码
     * */
    @PostMapping("/valid-register-email")
    public RestBean<String> validRegisterEmail(@Pattern(regexp = EMAIL_REGEXP)@RequestParam("email") String email,
                                               HttpSession session){
        String str = service.sendValidateEmail(email,session.getId(), false);
        if (str == null)
            return RestBean.success("邮件发送成功，请注意查收");
        else
            return RestBean.failure(400,str);
    }

    /*
     * 用户忘记密码发送邮箱验证码
     * */
    @PostMapping("/valid-reset-email")
    public RestBean<String> validResetEmail(@Pattern(regexp = EMAIL_REGEXP)@RequestParam("email") String email,
                                            HttpSession session){
        String str = service.sendValidateEmail(email,session.getId(), true);
        if (str == null)
            return RestBean.success("邮件发送成功，请注意查收");
        else
            return RestBean.failure(400,str);
    }

    /*
     * 新用户注册
     * */
    @PostMapping("/register")
    public RestBean<String> registerUser(@Pattern(regexp = USERNAME_REGEXP) @Length(min = 2,max = 8) @RequestParam("username") String userName,
                                         @Length(min = 6, max = 16) @RequestParam("password") String password,
                                         @Pattern(regexp = EMAIL_REGEXP) @RequestParam("email") String email,
                                         @Length(min = 6, max = 6) @RequestParam("code") String code,
                                         HttpSession session) {
        String str = service.vaildateAndRegister(userName, password, email, code, session.getId());
        if (str == null){
            return RestBean.success("注册成功，请前往登录页面进行登录");
        } else {
            return RestBean.failure(400,str);
        }
    }

    @PostMapping("/start-reset")
    public RestBean<String> startReset(@Pattern(regexp = EMAIL_REGEXP) @RequestParam("email") String email,
                                       @Length(min = 6, max = 6) @RequestParam("code") String code,
                                       HttpSession session) {
        String str = service.vaildateOnly(email, code, session.getId());
        if (str == null){
            session.setAttribute("reset-password", email);
            return RestBean.success();
        } else {
            return RestBean.failure(400, str);
        }
    }
    @PostMapping("/do-reset")
    public RestBean<String> ResetPassword(@Length(min = 6, max = 16) @RequestParam("password") String password,
                                          HttpSession session) {
        String email = (String) session.getAttribute("reset-password");
        if (email == null){
            return RestBean.failure(400,"请先完成邮箱验证");
        } else if (service.resetPassword(password,email)){
            session.removeAttribute("reset-password");
            return RestBean.success("密码重置成功");
        } else {
            return RestBean.failure(500,"内部错误，请联系管理员");
        }
    }

}
